#!/bin/sh
#
# $Id: collect_data_kvm.sh 15 2015-08-18 12:06:42Z zhongjinming@h3c.com $
#
# data collection script for SPECvirt_sc2013 under KVM
#
# TODO:
# handle physical client with no vclient with a graceful way
#

OUTPUT_DIR=/tmp/data-collection
CONFIG=/opt/SPECvirt/collect_data.cfg

# temp settings
SCRIPT_REMOTE=/tmp/collect_data.sh
BASEDIR_SYS=/tmp/data-collection-sys
BASEDIR_SW=/tmp/data-collection-sw

if echo "$0" | grep -q '^\/.*'
then
    SCRIPT_LOCAL="$0"
else
    SCRIPT_LOCAL="$(pwd)/$0"
fi

SSH_OPTIONS="-o IdentityFile=/autobench/ssh/autobench-id -o StrictHostKeyChecking=no"
echo "Gathering SUT data..."
if [ -e ~/README-SUT-notes.txt ]; then
    cp -p ~/README-SUT-notes.txt .
    fi
    cat_files='proc/cpuinfo proc/devices proc/meminfo proc/modules proc/partitions proc/swaps proc/mounts proc/version proc/cmdline'
    tar_files='/boot/grub/grub.conf /boot/efi/EFI/redhat/grub.conf /etc/fstab /etc/hosts /etc/inittab /etc/iproute2 /etc/nsswitch.conf /etc/sysctl.conf /etc/security/limits* /etc/modprobe* /etc/rc.local /etc/cgconfig.conf /etc/rc.d/* /etc/init.d/* /etc/sysconfig/network/* /etc/network/interfaces /etc/profile /var/log/dmesg /etc/kvm/vm-scripts/*t??.sh /etc/kvm/*ifup* /root/do_renice.sh'


sanity_check()
{
    data_collect_log "Checking hostname..." 3
    ping_host "sut"

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        ping_host "phyclient${client}"
    done

    for tile in $(seq $TILE_LIST)
    do
        ping_host "client${tile}" 
        ping_host "wclient${tile}" 

        vmlist="appserver batchserver infraserver mailserver webserver"
        rest=$[$tile%4]
        if [ "$rest" -eq 1 ]; then
            vmlist="$vmlist dbserver"
        fi

        for host in $vmlist
        do
            ping_host "${host}${tile}"
        done
    done

    return 0;
}

ping_host()
{
    local host="$1"
    echo ${host}
    if ! ping -w2 -c1 ${host} &>/dev/null
    then
        data_collect_log "${host} not pingable. Please check /etc/hosts!" 2
        exit 2
    fi
}

#
# not used yet
#
setup_host_key()
{
#    local CLIENT="$1"
#    local PASS="$2"

#    expect <<ENDEXP
#spawn ssh-copy-id -i /root/.ssh/id_rsa.pub root@${CLIENT}
#expect "*?assword:*"
#send -- "${PASS}\r"
#send -- "\r"
expect eof
ENDEXP
}

#
# this will setup the key auth
#
prepare_run_env()
{
    setup_host_key sut

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        setup_host_key "phyclient${client}"
    done

    for tile in $(seq $TILE_LIST)
    do
        setup_host_key "client${tile}" 

        vmlist="appserver batchserver infraserver mailserver webserver"
        rest=$[$tile%4]
        if [ "$rest" -eq 1 ]; then
            vmlist="$vmlist dbserver"
        fi

        for host in $vmlist
        do
            setup_host_key "${host}${tile}"
        done
    done
echo "hi"
}

build_dir_tree()
{
    while read DIR
    do
        mkdir -p $DIR
    done <<ENDLINE
./VIRT
./PHYS
./PHYS/SUT
./PHYS/Clients
./support_files
ENDLINE
}

data_collect_log()
{
    local text="$1"
    local class="$2"
    shift 2

    if [ "$class" = "2" ]
    then
        echo "$text" 1>&2
    elif [ "$class" = "3" ]
    then
        echo $* "$text"
    fi

    echo "$(date "+%b %d %Y %T") $text" >> $OUTPUT_DIR/data-collection.log
}

gen_list()
{
    local IFS=","
    local LIST="$1"

    for i in $LIST
    do
        if echo $i | grep -q '^[0-9]*-[0-9]*$'
        then
            seq ${i%-*} ${i#*-}
        else
            echo $i
        fi
    done
}

collect_proc_info()
{
    local procfile="$1"
    if [ ! -e "$procfile" ]
    then
        return 1
    fi

    local DIR=$(dirname $procfile)
    local FILE=$(basename $procfile)

    DIR=${DIR#/}
    #mkdir -p "$DIR" &>/dev/null
    mkdir -p $DIR

    if [ $? -ne 0 ]
    then
        return 1
    fi

    cat $procfile > "$DIR/$FILE"
}

collect_sys_info()
{
# hw and device info
    lspci > lspci.txt

# os info
#    hostname > hostname.txt

# network
    ifconfig > ifconfig.txt
    route -n > route.txt
    brctl show > brctl.txt 

# storage and filesystem
    collect_proc_info "/proc/partitions"
    vgdisplay -v >vgdisplay.txt 2>/dev/null
    lvdisplay > lvdisplay.txt 2>/dev/null
    mount > mount.txt
    df > df.txt

# packages and service
    rpm -qa > rpmpkgs.txt
    chkconfig --list > chkconfig.txt

# runtime info
    ps -ef | grep qemu > running-vms.txt
    virsh list > virsh-list.txt
    sysctl -a > sysctl.txt

# proc info
    collect_proc_info "/proc/cpuinfo"
    collect_proc_info "/proc/devices"
    collect_proc_info "/proc/meminfo"
    collect_proc_info "/proc/modules"
    collect_proc_info "/proc/swaps"
    collect_proc_info "/proc/version"
    collect_proc_info "/proc/cmdline"



# other static configuration and logs
find /etc/fstab \
/etc/redhat-release \
/etc/iproute2 \
/etc/hosts \
/etc/inittab \
/etc/sysctl.conf \
/etc/nsswitch.conf \
/etc/kvm \
/etc/security/limits.conf \
/etc/rc.d/init.d \
/etc/rc.d/rc3.d \
/opt/common_scripts/ostune_misc.sh \
/var/log/dmesg \
/boot/grub/grub.conf\
/boot/grub/grub.cfg \
/etc/bashrc \
/etc/exports \
/etc/rc.d/rc.local \
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
tar chf - $tar_files | tar xf -

# clear zero size file
find . -size 0 -exec rm -f {} \;

}

collect_remote_sys_info()
{
    local host="$1"
    
    echo $host

    scp -q $SCRIPT_LOCAL $host:$SCRIPT_REMOTE
    ssh $host sh $SCRIPT_REMOTE sysinfo
    ssh $host tar -cz /tmp/data-collection-sys 2>/dev/null | tar -xz 2>/dev/null
    mv tmp/data-collection-sys/* .
if [ "$host" = "sut" ]
then echo a
elif [ "$host" = "phyclient1" ]
then echo b
else
    rm -rf running-vms.txt
fi

rm -fr tmp
}

collect_SUT()
{
    cd SUT
    data_collect_log "Collecting SUT info..." 3
    data_collect_log "Entering $(pwd)..."

    collect_remote_sys_info "sut"

    cd ..
}

collect_clients()
{
    cd Clients
    data_collect_log "Collecting physical clients info..." 3
    data_collect_log "Entering $(pwd)..."

    for client in $(gen_list $PHYCLIENT_LIST)
    do
        dir="phyclient${client}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..." 
        collect_remote_sys_info $dir &
        cd ..
    done

    wait

    
    data_collect_log "Collecting virtual clients info..." 3
    for vclient in $(seq $TILE_LIST)
    do
        dir="client${vclient}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..."
        collect_remote_sys_info "client${vclient}" &
        cd ..

        dir="wclient${vclient}"
        mkdir $dir && cd $dir
        data_collect_log "Collecting $dir..."
        collect_remote_sys_info "wclient${vclient}" &
        cd ..
    done

    wait

    #data_collect_log "Linking virtual clients to physical clients..." 3
    #for client in $(gen_list $PHYCLIENT_LIST)
    #do
    #    dir="phyclient${client}"
    #    cd $dir
    #    for x in ${VCLIENT_LIST[$client]}
    #    do
    #        ln -s ../client${x} vclient${x}
    #        ln -s ../wclient${x} vwclient${x}
    #    done
    #    cd ..
    #done

    cd ..
}

collect_physical_configuration()
{
    cd PHYS
    data_collect_log "Entering $(pwd)..."

    collect_SUT
    collect_clients

    cd ..
}

collect_local_software_config()
{
    local type="$1"

    case $type in

        "appserver")
           java -version 2> javaVersion.out
            cp -p /opt/glassfish4/glassfish/domains/domain1/config/domain.xml .
        ;;
        "dbserver")
            mysql --version > myql-Version.txt
            cp -p /etc/my.cnf.d/my.cnf .
	    pushd /opt/SPECjAppServer2004/
	
            tar cf -	\
schema/mysql bin/glassfish  \
2> /dev/null | tar xf - -C $BASEDIR_SW &>/dev/null
        ;;

        "batchserver")
            java -version 2> javaVersion.out
            java -jar /opt/SPECpoll/pollme.jar -v > SPECpollVersion.out
        ;;

        "infraserver")
            sum /var/www/fcgi-bin/besim_fcgi.fcgi > sum.besim_fcgi
            ldd /var/www/fcgi-bin/besim_fcgi.fcgi > ldd.besim_fcgi

            find /etc/httpd/conf \
/etc/httpd/conf.d 	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
            httpd -V > etc/httpd/httpdVersion.out

        ;;

        "mailserver")
            find /etc/dovecot.conf \
/etc/dovecot \
/etc/init.d/dovecot 	\
2>/dev/null | cpio -o 2>/dev/null | cpio -idm --no-absolute-filenames &>/dev/null
            dovecot --version > etc/dovecotVersion.out
            dovecot --build-options >> etc/dovecotVersion.out
        ;;

        "webserver")
        nginx -v 2> ./nginxVersion.out  
        cp -p /etc/php.ini .
        cp -p /etc/nginx/nginx.conf .
        cp -p /etc/nginx/conf.d/default.conf .
        cp -p /etc/nginx/conf.d/example_ssl.conf .

        ;;

    esac
}

collect_remote_software_config()
{
    local host="$1"
    local type="$2"

    scp -q $SCRIPT_LOCAL $host:$SCRIPT_REMOTE
    ssh $host sh $SCRIPT_REMOTE "software" $type
    ssh $host tar -cz /tmp/data-collection-sw 2>/dev/null | tar -xz 2>/dev/null
    mv tmp/data-collection-sw/* .
    rm -fr tmp
}

collect_software_config()
{
    local type="$1"
    local tile="$2"
    local host="${type}${tile}"

    data_collect_log "Collecting software configuration on ${host}..."
    mkdir SOFTW && cd SOFTW
    collect_remote_software_config "$host" "$type" &
    cd ..
}

collect_vm_config()
{
    local host="$1"

    data_collect_log "Collecting vm configuration on ${host}..."
    mkdir VM && cd VM
    collect_remote_sys_info "$host" &
    cd ..
}

collect_gustxmls()
{
cd VIRT
pwd
mkdir -p guest-xml-config
pushd guest-xml-config >/dev/null
  for ntile in `seq 1 $TILE_LIST`; do
     zpt=`printf "%02d" $ntile`
       for vmtype in appserv batch infra mail web ; do
        if [ "$vmtype" = "db" ]; then
              rest=$[$ntile%4]
              if [ "$rest" != 1 ]; then continue
              fi
       fi
        scp sut:/etc/libvirt/qemuxml/${ntile}${vmtype}*.xml .
       done
  done
  scp sut:/etc/libvirt/qemuxml/1db*.xml ./1db.xml
  scp sut:/etc/libvirt/qemuxml/5db*.xml ./5db.xml
  popd >/dev/null
cd ..
                                                                            
}

collect_one_tile()
{
    local tile="$1"
    local host=""

    vmlist="appserver batchserver infraserver mailserver webserver"
    rest=$[$tile%4]


    if [ "$rest" -eq 1 ]; then
        vmlist="$vmlist dbserver"
    fi


    for host in $vmlist
    do
        mkdir $host && cd $host
        #collect KVM xml for tile
	

        data_collect_log "Collecting ${host} in tile${tile}..."

        collect_software_config "$host" "$tile"
        collect_vm_config "${host}${tile}"

        cd ..
    done

    # per tile wait
    #wait
}

collect_support_files()
{
	cd support_files
        cp /opt/SPECvirt/collect_data.cfg ./
        cp /opt/SPECvirt/collect_data.sh ./
        cp /opt/SPECvirt/setMTU.sh ./
	cd ..
}

collect_virtual_configuration()
{
    cd VIRT
    data_collect_log "Entering $(pwd)..."

    for tile in $(seq $TILE_LIST)
    do
        mkdir "tile${tile}" && cd "tile${tile}"
        data_collect_log "Collecting tile${tile}..." 3
        data_collect_log "Entering $(pwd)..."
        collect_one_tile "${tile}"
        cd ..
    done

    cd ..
}

#
# main routine starts here
#

if [ "$1" = "sysinfo" ]
then
    rm -fr $BASEDIR_SYS 2>/dev/null && mkdir $BASEDIR_SYS 2>/dev/null && cd $BASEDIR_SYS
    collect_sys_info
elif [ "$1" = "software" ]
then
    rm -fr $BASEDIR_SW 2>/dev/null && mkdir $BASEDIR_SW 2>/dev/null && cd $BASEDIR_SW
    collect_local_software_config "$2"
elif [ "$1" = "prepare" ]
then
    prepare_run_env
else

    if [ -r "$CONFIG" ]
    then
        source $CONFIG
    else
        data_collect_log "Config $CONFIG not readable, please check!" 2
        exit 1
    fi

    rm -fr $OUTPUT_DIR 2>/dev/null && mkdir $OUTPUT_DIR 2>/dev/null && cd $OUTPUT_DIR
    sanity_check

    build_dir_tree

    date > Data-Collection-Start.txt

    collect_physical_configuration


    collect_virtual_configuration
    
    collect_gustxmls  
    
    # final wait
    wait
    collect_support_files

    date > Data-Collection-End.txt

    data_collect_log "Data has been collected under $OUTPUT_DIR" 3

    mv data-collection.log /tmp/

    exit 0
fi

